
<aside class="sidebar">
    <div class="sidebar-header">
        <div class="logo">
            <img src="images/logo.svg" alt="Metripy Logo" class="logo-icon">
            <h2>Metripy</h2>
        </div>
    </div>
    
    <nav class="sidebar-nav">
        <ul>
            <li class="nav-item">
                <a href="index.html" class="nav-link {{#IF sidebar_active_index}}active{{/IF}}">
                    <i class="fas fa-tachometer-alt"></i>
                    <span>Overview</span>
                </a>
            </li>
            <li class="nav-item">
                <a href="files.html" class="nav-link {{#IF sidebar_active_files}}active{{/IF}}">
                    <i class="fas fa-file-code"></i>
                    <span>Files</span>
                </a>
            </li>
            <li class="nav-item">
                <a href="top_offenders.html" class="nav-link {{#IF sidebar_active_top_offenders}}active{{/IF}}">
                    <i class="fa-solid fa-stethoscope"></i>
                    <span>Top Offenders</span>
                </a>
            </li>
            <li class="nav-item">
                <a href="git_analysis.html" class="nav-link {{#IF sidebar_active_git_analysis}}active{{/IF}}">
                    <i class="fab fa-git-alt"></i>
                    <span>Git Analysis</span>
                </a>
            </li>
            <li class="nav-item">
                <a href="coupling.html" class="nav-link {{#IF sidebar_active_coupling}}active{{/IF}}">
                    <i class="fas fa-project-diagram"></i>
                    <span>Coupling</span>
                </a>
            </li>
            <li class="nav-item">
                <a href="dependencies.html" class="nav-link {{#IF sidebar_active_dependencies}}active{{/IF}}">
                    <i class="fas fa-cubes"></i>
                    <span>Dependencies</span>
                </a>
            </li>
            <li class="nav-item">
                <a href="trends.html" class="nav-link {{#IF sidebar_active_trends}}active{{/IF}}">
                    <i class="fas fa-chart-line"></i>
                    <span>Trends</span>
                </a>
            </li>
            <li class="nav-item">
                <a href="code_smells.html" class="nav-link {{#IF sidebar_active_code_smells}}active{{/IF}}">
                    <i class="fas fa-bug"></i>
                    <span>Code Smells</span>
                </a>
            </li>
        </ul>
    </nav>
    
    <div class="sidebar-footer">
        <div class="project-info">
            <h4>Project: {{project_name}}</h4>
            <p>Last updated: {{last_updated}}</p>
        </div>
    </div>
</aside>